<?php

namespace v2\Models;

use Config;
use controller;
use Filters\Traits\Filterable;
use Illuminate\Database\Capsule\Manager as DB;
use v2\Filters\Filters\BetcodeConversionFilter;
use Illuminate\Database\Eloquent\Model as Eloquent;


class BetcodeConversion extends Eloquent
{
    use Filterable;


    protected $fillable = [
        'user_id',
        'home_bookie_id',
        'dest_bookie_id',
        'bookies_train',
        'booking_code',
        'home_entries',
        'dest_entries',
        'destination_code',
        'proxy',
        'dump',
        'channel',
        'status',
    ];

    protected $table = 'betcodes_conversions';


    public static $query_config = [
        'filter_class' => BetcodeConversionFilter::class, //the filter class 
        'pass_mark' => 1,
        'name' => 'conversions', //variable name for this
    ];


    /* 
        statuses
        1---gotten home entries
        2---ongoing chunk conversion
        3--- postable payload ready to destination bookie
        4---attempted
    */


    public function toTrimmedArray()
    {
        $conversion = $this->toArray();
        $unsets = [
            'home_bookie_id',
            'dest_bookie_id',
            'home_entries',
            'dest_entries',
            'proxy',
            'id',
            'user_id',
        ];

        foreach ($unsets as $key) {
            unset($conversion[$key]);
        }

        unset($conversion['dump']['home_bookie_entries']['raw_events']);
        unset($conversion['dump']['home_bookie_entries']['exempted_events']);
        unset($conversion['dump']['home_bookie_entries']['uniform_event']);

        unset($conversion['dump']['destination_bookie_entries']['postable_lines']);
        unset($conversion['dump']['destination_bookie_entries']['converted_booking']['raw_events']);
        unset($conversion['dump']['destination_bookie_entries']['response_array']);


        foreach ($conversion['dump']['home_bookie_entries']['found_events'] as $key => $value) {
            unset($conversion['dump']['home_bookie_entries']['found_events'][$key]['translated_prediction']);
        }

        foreach ($conversion['dump']['destination_bookie_entries']['converted_booking']['uniform_event'] as $key => $value) {
            unset($conversion['dump']['destination_bookie_entries']['converted_booking']['uniform_event'][$key]['translated_prediction']);
        }

        return $conversion;
    }

    public function stateOfConversion()
    {

        $retry = (int)(($this->destination_code == null || !is_array($this->dump)) && $this->status == 4);

        $complete = (int)(($this->destination_code != null && is_array($this->dump)) && $this->status == 4);


        if ($complete || $this->status <= 2) {
            return null;
        }

        if ($this->status == 3) {
            $retry = 3;
        }


        $array = [
            '1' => 'retry_post',
            '3' => 'retry_post',
            '0' => "retry_conversion"
        ];

        return $array[$retry] ?? null;
    }

    public function isOk()
    {
        return !(($this->destination_code == null || $this->dump == 0) && $this->status == 4);
    }

    public function getlinkAttribute()
    {
        $domain = Config::domain();
        $auth = (new controller)->auth();


        $conversion = $this;
        $slug = "{$conversion['home_bookie']['name']}-code-{$conversion['dump']['home_bookie_entries']['summary']['booking_code']}-converted-to-{$conversion['destination_bookie']['name']}";

        if ($auth) {
            $link = "$domain/c/$this->id/$slug";
        } else {
            $link = "$domain/login";
        }

        // $link = "$domain/c/$this->id";
        return $link;
    }

    public static function TopMostConvertedCode()
    {
        $today = date("Y-m-d");

        $conversions =  self::select('booking_code', 'home_bookie_id', DB::raw('count(*) freq'))
            ->whereDate('updated_at', "$today")
            // ->Attempted()
            ->groupBy('booking_code', 'home_bookie_id')
            ->orderBy('freq', 'DESC')
            ->take(10);

        $total = $conversions->get()->sum('freq');

        $bookies = BookMaker::whereIn('id', $conversions->get()->pluck('home_bookie_id')->toArray())->get()->keyBy('id');

        $conversions = $conversions->get()->map(function ($item) use ($total, $bookies) {
            $item['percent'] = round(($item['freq'] / $total) * 100);
            $item['bookie'] = $bookies[$item['home_bookie_id']]['name'];
            return $item;
        });



        return $conversions;
    }

    public function home_bookie()
    {
        return $this->belongsTo(BookMaker::class, 'home_bookie_id');
    }

    public function destination_bookie()
    {
        return $this->belongsTo(BookMaker::class, 'dest_bookie_id');
    }

    public function scopeAttempted($query)
    {
        return $query->where('status', 4);
    }

    public function is_attempted()
    {
        return $this->status == 4;
    }

    public function markAsAttempted()
    {
        $this->update(['status' => 4]);
    }

    public function getHomeEntriesArrayAttribute()
    {
        return $this->home_entries;
    }

    public function getslugAttribute()
    {
        return $this->attributes['slug'] =  "this-is-th-slug";
    }

    public function getHomeEntriesAttribute($value)
    {
        if ($value == null) {
            return [];
        }

        return json_decode($value, true);
    }


    public function getDestEntriesArrayAttribute()
    {
        return   $this->dest_entries;
    }


    public function getDestEntriesAttribute($value)
    {
        if ($value == null) {
            return [];
        }

        return json_decode($value, true);
    }


    public function getDetailsArrayAttribute()
    {
        return $this->dump;
    }

    public function markAsPostableToDestinationBookie()
    {
        $this->update(['status' => 3]);
    }

    public function getDumpAttribute($value)
    {
        if ($value == null) {
            return [];
        }

        return json_decode($value, true);
    }

    public function getProxyAttribute($value)
    {
        if ($value == null) {
            return [];
        }

        return json_decode($value, true);
    }
}
