<?php

namespace v2\Models\Wallet;


use Exception, User;
use v2\Classes\ExchangeRate;
use v2\Models\Wallet\Journals;
use v2\Models\Wallet\BasicAccountType;
use v2\Models\Wallet\CompanyAccountType;
use v2\Models\Wallet\JournalInvolvedAccounts;
use Illuminate\Database\Capsule\Manager as DB;
use v2\Models\Wallet\AccountClosingAndReopening;
use Illuminate\Database\Eloquent\Model as Eloquent;

class ChartOfAccount extends Eloquent
{

	protected $fillable = [
		'basic_account_type_id',
		'company_customised_account_id',
		'owner_id',
		'user_id',
		'company_id',
		'opening_balance',
		'current_balance',
		'currency',
		'account_name',
		'account_code',
		'description',
		'on_watchlist',
		'is_subaccount'
	];

	protected $table = 'ac_charts_of_accounts';
	protected $connection = 'wallet';

	public static $base_currency = 'GHC';

	/**
	 * [$validator_rules this contains validations rules.
	 * it will be used with the Validator classs.
	 * keys are names of form inputs POSTED while values are arrays 
	 * containing rules]
	 * @var [array]
	 */
	public static $validator_rules = [
		'opening_balance' => [
			'numeric' =>  true,
		],

		'account_type' => [
			'required' =>  true,
		],

		'username' => [
			'exist' => User::class . "|username",
		],

		/* 'basic_account_type_id' => [
			'max' =>  500,
		], */

		'account_name' => [
			'required' =>  true,
		],


	];



	public function getConversion($amount)
	{
		$base_currency = self::$base_currency;
		$exchange = new ExchangeRate;
		$conversion = $exchange->setFrom($this->currency)
			->setTo($base_currency)
			->setAmount($amount)
			->getConversion();
		return $conversion;
	}

	public function getBaseOpeningBalanceAttribute()
	{
		$conversion = $this->getConversion($this->opening_balance);
		$destination = $conversion['destination_value'];
		return $destination;
	}

	public function getBaseCurrentBalanceAttribute()
	{
		$conversion = $this->getConversion($this->current_balance);
		$destination = $conversion['destination_value'];
		return $destination;
	}


	public function owner()
	{
		return $this->belongsTo(User::class, 'owner_id');
	}

	public function subtract_opening_balance($amount)
	{

		try {

			$this->decrement('opening_balance', $amount);
			$this->decrement('current_balance', $amount);
			$transactions = $this->all_transaction();

			foreach ($transactions as  $transaction) {

				$transaction->decrement('prior_balance', $amount);
				$transaction->decrement('post_balance', $amount);
			}

			return true;
		} catch (Exception $e) {
			return false;
		}
	}

	public function add_to_opening_balance($amount)
	{
		try {

			$this->increment('opening_balance', $amount);
			$this->increment('current_balance', $amount);
			$transactions = $this->all_transaction();

			foreach ($transactions as  $transaction) {

				$transaction->increment('prior_balance', $amount);
				$transaction->increment('post_balance', $amount);
			}

			return true;
		} catch (Exception $e) {
			return false;
		}
	}



	public function all_transaction($from = null, $to = null)
	{



		$sql = Journals::CompanyJournals($this->company_id)
			->Published();


		$chart_of_account_id = $this->id;
		$activated_period = AcBooksSettings::activated_period();
		switch ($from) {
			case null:
				$from = $activated_period->start_date;
				break;

			default:
				# code...
				break;
		}

		switch ($to) {
			case null:
				$to = $activated_period->end_date;
				break;

			default:
				# code...
				break;
		}

		$sql = $sql->DateRange($from, $to);

		$journals =  $sql->get();

		$journals_ids_sorted_by_date = $journals->pluck('id')->toArray();

		$journals_ids_by_chart_of_account = JournalInvolvedAccounts::where('chart_of_account_id', $this->id)
			->get()->pluck('journal_id')
			->toArray();


		$journals_ids = array_unique(array_intersect($journals_ids_by_chart_of_account, $journals_ids_sorted_by_date));




		$transactions = JournalInvolvedAccounts::whereIn('journal_id', $journals_ids)
			->where('chart_of_account_id', $this->id)
			->get();



		return $transactions;
	}


	public function is_open()
	{
		return	AccountClosingAndReopening::get_state_boolean($this->id);
	}


	public function getOpenOrCloseStatusAttribute()
	{
		if ($this->is_open()) {
			return  "<span class='badge badge-success' >Open</span>";
		} else {

			return  "<span class='badge badge-danger' >Close</span>";
		}
	}

	public function post_credit($amount)
	{
		if ($this->is_credit_balance()) {

			$this->increment('current_balance', $amount);
		} else {

			$this->decrement('current_balance', $amount);
		}

		return $this->current_balance;
	}


	public function post_debit($amount)
	{
		if ($this->is_debit_balance()) {

			$this->increment('current_balance', $amount);
		} else {

			$this->decrement('current_balance', $amount);
		}

		return $this->current_balance;
	}


	public static function account_format($value)
	{
		if ($value < 0) {
			return  "(" . number_format(abs($value), 2) . ")";
		} else {

			return number_format($value, 2);
		}
	}



	public function is_credit_balance()
	{
		return ($this->account_type->is_credit_balance());
	}



	public function is_debit_balance()
	{
		return ($this->account_type->is_debit_balance());
	}




	public function transactions($from = null, $to = null, $per_page = 10, $page = 1)
	{
		$sql = Journals::CompanyJournals($this->company_id)
			->Published();


		$chart_of_account_id = $this->id;
		$activated_period = AcBooksSettings::activated_period();
		switch ($from) {
			case null:
				$from = $activated_period->start_date;
				break;

			default:
				# code...
				break;
		}

		switch ($to) {
			case null:
				$to = $activated_period->end_date;
				break;

			default:
				# code...
				break;
		}

		$sql = $sql->DateRange($from, $to);

		$journals =  $sql->get();

		$journals_ids_sorted_by_date = $journals->pluck('id')->toArray();

		$journals_ids_by_chart_of_account = JournalInvolvedAccounts::where('chart_of_account_id', $this->id)
			->get()->pluck('journal_id')
			->toArray();


		$journals_ids = array_unique(array_intersect($journals_ids_by_chart_of_account, $journals_ids_sorted_by_date));

		$skip = ($page - 1) * $per_page;



		$transactions = JournalInvolvedAccounts::whereIn('journal_id', $journals_ids)
			->where('chart_of_account_id', $this->id)
			->offset($skip)
			->take($per_page)
			->get();



		return $transactions;
	}


	/**
	 * [last_post description]
	 * @param  [date] $as_of [date]
	 * @return [JournalInvolvedAccounts] - this is the last entry posted to this chart of account 
	 * on which we can get the last_post
	 *         
	 */

	public function last_post($from = null, $as_of)
	{
		return $this->posts($from, $as_of, 3)->first();
	}

	public function posts($from = null, $to = null, $per_page = 10, $page = 1)
	{
		$sql = Journals::CompanyJournals($this->company_id)
			->Published();


		//include financial year code to set default date range



		$sql = $sql->DateRange($from, $to);

		$journals =  $sql->get();

		$journals_ids_sorted_by_date = $journals->pluck('id')->toArray();

		$journals_ids_by_chart_of_account = JournalInvolvedAccounts::where('chart_of_account_id', $this->id)
			->get()->pluck('journal_id')
			->toArray();


		$journals_ids = array_unique(array_intersect($journals_ids_by_chart_of_account, $journals_ids_sorted_by_date));

		$skip = ($page - 1) * $per_page;



		$balances = JournalInvolvedAccounts::whereIn('journal_id', $journals_ids)
			->where('chart_of_account_id', $this->id)
			->offset($skip)
			->latest('updated_at')
			->take($per_page)
			//thi is to be reviewed because balances 
			// can be by the last updated(updated_at) or last entered(id)
			->get();



		return $balances;
	}

	public function posts_without_pagination($from = null, $to = null)
	{
		$sql = Journals::CompanyJournals($this->company_id)
			->Published();

		$sql = $sql->DateRange($from, $to);

		$journals =  $sql->get();

		$journals_ids_sorted_by_date = $journals->pluck('id')->toArray();

		$journals_ids_by_chart_of_account = JournalInvolvedAccounts::where('chart_of_account_id', $this->id)
			->get()->pluck('journal_id')
			->toArray();


		$journals_ids = array_unique(array_intersect($journals_ids_by_chart_of_account, $journals_ids_sorted_by_date));



		$posts = JournalInvolvedAccounts::whereIn('journal_id', $journals_ids)
			->where('chart_of_account_id', $this->id)
			->latest('updated_at')
			//thi is to be reviewed because balances 
			// can be by the last updated(updated_at) or last entered(id)
			->get();



		return $posts;
	}


	public static function for_company($company_id)
	{
		return self::where('company_id', $company_id);
	}


	/**
	 * 
	 * the result is used at chart of  account creation page
	 * to populate the account select options for  account type
	 */
	public static function charts_of_account_options_at_creation($company_id)
	{


		$charts = CompanyAccountType::for_company($company_id)
			->get()->groupBy('basic_account_id');


		foreach ($charts as $basic_account_type_id => $charts_of_accounts) {
			$basic_name = BasicAccountType::find($basic_account_type_id)->name;
			foreach ($charts_of_accounts as $key => $chart_of_account) {


				$refined_output[$basic_name][] = $chart_of_account->toArray();
			}
		}

		return ($refined_output);
	}
	/**
	 * 
	 * the result is used at chart of  account settings page
	 * to populate the account select options for  accounts type
	 */
	public static function charts_of_account_options_at_edit($company_id, $chart_of_account_id)
	{


		$charts = CompanyAccountType::for_company($company_id)
			->get()->groupBy('basic_account_id');

		$chart_of_account_in_edit = ChartOfAccount::find($chart_of_account_id);


		foreach ($charts as $basic_account_type_id => $charts_of_accounts) {
			$basic_account = BasicAccountType::find($basic_account_type_id);
			$basic_name =  $basic_account->name;

			if (($basic_account->is_credit_balance()) !=
				($chart_of_account_in_edit->account_type->is_credit_balance())
			) {
				continue;
			} elseif (($basic_account->is_debit_balance()) !=
				($chart_of_account_in_edit->account_type->is_debit_balance())
			) {
				continue;
			}

			foreach ($charts_of_accounts as $key => $chart_of_account) {

				$refined_output[$basic_name][] = $chart_of_account->toArray();
			}
		}

		return ($refined_output);
	}

	/**
	 * 
	 * the result is used at journal creation page
	 * to populate the account select options for involved accounts
	 */
	public static function charts_of_account_options($company_id)
	{


		$charts = ChartOfAccount::for_company($company_id)
			->get()->groupBy('basic_account_type_id');


		foreach ($charts as $basic_account_type_id => $charts_of_accounts) {
			$basic_name = BasicAccountType::find($basic_account_type_id)->name;
			foreach ($charts_of_accounts as $key => $chart_of_account) {

				if ($chart_of_account->is_open()) {

					$refined_output[$basic_name][] = $chart_of_account->toArray();
				}
			}
		}
		return ($refined_output);
	}


	public static function generate_account_code($company_id, $basic_account_type_id)
	{

		$count = self::for_company($company_id)->where('basic_account_type_id', $basic_account_type_id)->count();
		$code = ($basic_account_type_id *  1000) + $count + 1;
		return $code;
	}






	public function custom_account_type()
	{
		return $this->belongsTo(CompanyAccountType::class, 'company_customised_account_id')->with(['basic_account']);
	}



	/**
	 * to be deleted
	 *
	 * @return void
	 */
	public function account_type()
	{
		return $this->belongsTo(BasicAccountType::class, 'basic_account_type_id');
	}
}
